/***************************************************************/
//
//
//		DirectX	[ fieldmap.cpp ]
//
//											Author	kazuki tanaka
//											Date	2017 5/29
/*---------------------------------------------------------------
Update : 2017/5/29
			fieldmap.cpp̍쐬

/*---------------------------------------------------------------
	Include File
---------------------------------------------------------------*/

#include "system_component.h"

#include "fieldmap.h"

#include "gameobject.h"
#include "base.h"

#include "game_equation.h"

#include "castdx9.h"

/*---------------------------------------------------------------
	Macro Difinition
---------------------------------------------------------------*/


/*---------------------------------------------------------------
	Const Number Difinition
---------------------------------------------------------------*/


/*----------------------------------------------------------------
	static member variable Initialize 
----------------------------------------------------------------*/

FieldMap* FieldMap::pMap = nullptr;




// FieldMap Constructor
FieldMap::FieldMap( )
	: pObj(nullptr)
	, map(nullptr)
	, vertical(0)
	, horizonal(0)
{

	// Menber Clear Process
	if( pMap ){
		delete pMap;
	}
	pMap = this;

}

// FieldMap Destructor
FieldMap::~FieldMap( )
{

	// Do Nothing!!

}

// FieldMap Create Instance 
FieldMap* FieldMap::Create( void )
{

	// Create Class Instance
	return new FieldMap;

}

// FieldMap Initialize
void FieldMap::Init( const char* filename )
{

	// Initialize Process
	if( !pMap ){
		return;
	}
	
	vertical  = 9;
	horizonal = 9;
	
	map  = new int[(vertical)*(horizonal)];
	pObj = new GameObject*[ vertical*horizonal ];

	for(int z = 0; z < pMap->horizonal; z++ ){
		for(int x = 0; x < pMap->vertical; x++ ){
			map[z*(vertical) + x]  = NONE;
			pObj[z*(vertical) + x] = nullptr;
		}
	}


	GridVertex* pVtx;

	// Create Vertex Process
	if( FAILED( GetDevice()->CreateVertexBuffer(
		sizeof( GridVertex ) * (((horizonal+1)+(vertical+1)))*2,		// mۂobt@TCY(_m) P:oCg
			D3DUSAGE_WRITEONLY,								// gp@
			GRID_FVF_FORMAT,								// gp钸_tH[}bg
			D3DPOOL_MANAGED,								// obt@̊Ǘ@
			&pVtxBuf,										// i[|C^
			NULL )))
	{
		#ifdef _DEBUG
		MessageBox( NULL, "obt@ւ݂̏Ɏs܂!!" , " DRAWLOGO VTXBUF ERROR !!" , MB_OK );
		#endif	// _DEBUG

		return;
	}

	pVtxBuf -> Lock(0,0,(void**)&pVtx,0);		

	const int size = MAP_SIZE;
	Vector2 offset( -(size*pMap->vertical)*0.5f, size*pMap->horizonal*0.5f );

	int index = 0;
	for( int z = 0; z < horizonal+1; z++ ){

		pVtx[index++].pos = D3DXVECTOR3( offset.x, 0,offset.y-size*z );
		pVtx[index].color = D3DXCOLOR(0.3, 1.0, 0.3, 1.0 );

		pVtx[index++].pos = D3DXVECTOR3( -offset.x,0,offset.y-size*z );
		pVtx[index].color = D3DXCOLOR(0.3, 1.0, 0.3, 1.0 );
	}

	for( int x = 0; x < vertical+1; x++  ){
			
		pVtx[index++].pos = D3DXVECTOR3( offset.x+size*x,0,offset.y );
		pVtx[index].color = D3DXCOLOR(0.3, 1.0, 0.3, 1.0 );

		pVtx[index++].pos = D3DXVECTOR3( offset.x+size*x,0,-offset.y );
		pVtx[index].color = D3DXCOLOR(0.3, 1.0, 0.3, 1.0 );
	}

	pVtxBuf ->Unlock();

}

// FieldMap Uninitialize
void FieldMap::Uninit( void )
{

	// Uninitialize Process
	SAFE_DELETE_ARRAY(map)
	delete pMap;

}

// FieldMap Update
void FieldMap::Update( void )
{

	// Update Process
	if( !pMap )
		return;

	// Update Process
	for(int z = 0; z < pMap->horizonal; z++ ){
		for(int x = 0; x < pMap->vertical; x++ ){
			map[z*(vertical) + x]  = NONE;
			pObj[z*(vertical) + x] = nullptr;
		}
	}



}

// FieldMap Draw
void FieldMap::Draw( void )
{

	// Draw Process
	if( !pMap )
		return;

	Matrix mat;

	mat = mat.Identity();

	GetDevice()->SetTransform( D3DTS_WORLD, &CastMtx(mat) );

	// Xg[
	GetDevice()->SetStreamSource(0,					
		pVtxBuf,								
		0,													
		sizeof( GridVertex ));						

	// _tH[}bg̐ݒ
	GetDevice()->SetFVF( GRID_FVF_FORMAT );

	// Draw Process
	GetDevice()->DrawPrimitive( D3DPT_LINELIST, 0, ((horizonal+1)+(vertical+1)) );

}

// FieldMap
void FieldMap::SetObject( Vector3 pos, TYPE type, GameObject* pObj )
{

	const int size = MAP_SIZE;
	Vector2 start( -(size*pMap->vertical)*0.5f, size*pMap->horizonal*0.5f );
	for(int z = 0; z < pMap->horizonal; z++ ){
		for(int x = 0; x < pMap->vertical; x++ ){

			Vector3 current = Vector3( start.x + size*x, 0, start.y - size*z );
			Vector3 next = Vector3( start.x + size*(x+1), 0, start.y - size*(z+1));

			Vector3 center = current + (next - current)*0.5f;

			//if( __CollisionObjectSquare<Vector3>( current, next, pos, size*0.5f ) ){
			if( __CollisionObjectSphere( &CastVec3(center), &CastVec3(pos), size*0.5f, size*0.5f )){
				if( !pMap->map[z*(pMap->vertical)+x] ){
					pMap->map[z*(pMap->vertical)+x]  = type;
					pMap->pObj[z*(pMap->vertical)+x] = pObj;
					CConsole::DebugLog(" object[%p] x:%d, z:%d \n",pObj, z, x );
					break;
				}else{
			
					pObj->pos = dynamic_cast<Base*>(pObj)->prev;
					SetObject( pObj->pos, type, pObj );
					break;
				}
			}
		}
	}

}

// FieldMap
void FieldMap::FieldCheck( void )
{


	CDebugFont::Access()->SetRenderData( R_RENDER ,"map data \n");

	for(int z = 0; z < pMap->horizonal; z++ ){
		for(int x = 0; x < pMap->vertical; x++ ){

			CDebugFont::Access()->SetRenderData( R_RENDER ," %d", map[z*pMap->vertical+x]);

			if( map[z*pMap->vertical+x] != NONE ){
				switch( map[z*(pMap->vertical)+x] )
				{
				case IMMORTAL:
					break;
				case MOVE:
					OnScissor( z, x );
					Scissor( z, x );
					break;
				case GOAL:
					OnScissor( z, x );
					break;
				default:
					break;
				}
			}
		}
		CDebugFont::Access()->SetRenderData( R_RENDER ,"\n");
	}

}

// FieldMap
int* FieldMap::GetStatus( void )
{

	return pMap->map;

}

// FieldMap
bool FieldMap::Scissor( const int& z, const int& x )
{

	if( x <= 1 || z <= 1 || x >= (vertical+1) || z >= (horizonal+1) ){
		return false;
	}

	if( map[z*(pMap->vertical)+x] != NONE && map[z*(pMap->vertical)+x-1] != NONE && map[z*(pMap->vertical)+x-2] != NONE ){
		if( pObj[z*(pMap->vertical)+x] )
			dynamic_cast<Base*>(pObj[z*(pMap->vertical)+x])->Scissor();
		return true;
	}
	if( map[z*(pMap->vertical)+x] != NONE && map[z*(pMap->vertical)+x+1] != NONE && map[z*(pMap->vertical)+x+2] != NONE ){
		if( pObj[z*(pMap->vertical)+x] )
			dynamic_cast<Base*>(pObj[(z)*(pMap->vertical)+x])->Scissor();
		return true;
	}
	if( map[z*(pMap->vertical)+x] != NONE && map[(z-1)*(pMap->vertical)+x] != NONE && map[(z-2)*(pMap->vertical)+x] != NONE ){
		if( pObj[z*(pMap->vertical)+x] )
			dynamic_cast<Base*>(pObj[(z)*(pMap->vertical)+x])->Scissor();
		return true;
	}
	if( map[z*(pMap->vertical)+x] != NONE && map[(z+1)*(pMap->vertical)+x] != NONE && map[(z+2)*(pMap->vertical)+x] != NONE ){
		if( pObj[z*(pMap->vertical)+x] )
			dynamic_cast<Base*>(pObj[(z)*(pMap->vertical)+x])->Scissor();
		return true;
	}

	return false;

}

// FieldMap
bool FieldMap::OnScissor( const int& z, const int& x )
{



	if( map[z*(pMap->vertical)+x+1] != NONE && map[z*(pMap->vertical)+x-1] != NONE ){
		dynamic_cast<Base*>(pObj[z*(pMap->vertical)+x])->OnScissor();
		return true;
	}
	if( map[(z+1)*(pMap->vertical)+x] != NONE && map[(z-1)*(pMap->vertical)+x] != NONE ){
		dynamic_cast<Base*>(pObj[z*(pMap->vertical)+x])->OnScissor();
		return true;
	}

	return false;


}






